@@ -60,6 +60,8 @@ def main_impl(argv, env_accessor=RealEnvAccessor(), printer=RealPrinter(), file_
   add_common_arguments(systemTestArgParser)
   systemTestArgParser.add_argument('app', metavar='App', type=str,
                              help="The name of the app to run the system tests for.")
+  systemTestArgParser.add_argument('--reuse-cluster', dest='reuseCluster', type=str, required=False, default=None,
+                             help="Optionally reuse an existing kind-cluster for system-tests. The cluster will be created if it does not yet exist.")
 
   def print_help(argv):
     print('HELP: TODO')
@@ -115,7 +117,7 @@ def main_impl(argv, env_accessor=RealEnvAccessor(), printer=RealPrinter(), file_
   def systemTest(argv):
     args = systemTestArgParser.parse_args(argv)
     kubedev = Kubedev()
-    return kubedev.system_test(args.config, args.app)
+    return kubedev.system_test(args.config, args.app, args.reuseCluster)
 
   commands = {
       'generate': generate,
