@@ -405,10 +405,10 @@ class Kubedev:
     e = env_accessor.getenv('KUBEDEV_KUBECONTEXT')
     return f'--kube-context {e}' if e != None and isinstance(e, str) and e != '' else ' '
 
-  def _template(self, kubedev, shell_executor, env_accessor, file_accessor, get_output=False):
+  def _template(self, kubedev, variable_overrides, shell_executor, env_accessor, file_accessor, get_output=False):
     variables = KubedevConfig.get_global_variables(kubedev)
     tag = KubedevConfig.get_tag(env_accessor)
-    envs = KubedevConfig.get_helm_set_env_args(kubedev, env_accessor)
+    envs = KubedevConfig.get_helm_set_env_args(kubedev, env_accessor, variable_overrides=variable_overrides, add_unknown_variable_overrides=True)
     command = [
         '/bin/sh',
         '-c',
@@ -428,15 +428,33 @@ class Kubedev:
       tag: str,
       release_name: str,
       docker_network: str,
+      variable_overrides: dict,
       shell_executor: object,
       env_accessor: object,
       file_accessor: object,
+      helm_chart: str = 'helm-chart/',
+      add_unknown_variable_overrides: bool = False,
       get_output: bool = False):
     variables = KubedevConfig.get_global_variables(kubedev)
     if docker_network is not None:
       kubeContext = env_accessor.getenv('KUBEDEV_KUBECONTEXT')
-      envs = KubedevConfig.get_helm_set_env_args(kubedev, env_accessor, cmdline_as_list=True)
+      envs = KubedevConfig.get_helm_set_env_args(
+        kubedev,
+        env_accessor,
+        variable_overrides=variable_overrides,
+        add_unknown_variable_overrides=add_unknown_variable_overrides,
+        cmdline_as_list=True)
+      normalized_kubeconfig = KubedevConfig.wsl_normalize(kubeconfig, file_accessor, shell_executor)
+      print()
+      print(f'{colorama.Fore.YELLOW}  (ℹ) kubedev is about to deploy your helm chart {helm_chart}.')
+      print(f'{colorama.Fore.YELLOW}  (ℹ) If you want to inspect what is going on in your cluster, use this command:')
+      print()
+      print(f'  docker run -it --rm --network {docker_network} --volume "{normalized_kubeconfig}:/tmp/kube_config" bitnami/kubectl --kubeconfig /tmp/kube_config <your-commands>')
+      print()
       command = [
+        "/bin/sh",
+        "-c",
+        " ".join([
           'docker',
           'run',
           '-i',
@@ -444,9 +462,9 @@ class Kubedev:
           '--network',
           docker_network,
           '--volume',
-          f'{KubedevConfig.wsl_normalize(kubeconfig, file_accessor, shell_executor)}:/tmp/kube_config',
+          f'{normalized_kubeconfig}:/tmp/kube_config',
           '--volume',
-          f'{KubedevConfig.wsl_normalize("helm-chart/", file_accessor, shell_executor)}:/app/helm-chart/',
+          f'{KubedevConfig.wsl_normalize(helm_chart, file_accessor, shell_executor)}:/app/helm-chart/',
           'alpine/helm:2.16.9',
           'upgrade',
           release_name,
@@ -457,13 +475,14 @@ class Kubedev:
           '/tmp/kube_config'] + \
           (["--kube-context", kubeContext] if kubeContext != None else []) + \
           ['--set', f'KUBEDEV_TAG={tag}'] + \
-          envs['cmdline']
+          envs['cmdline'])
+        ]
     else:
-      envs = KubedevConfig.get_helm_set_env_args(kubedev, env_accessor, cmdline_as_list = False)
+      envs = KubedevConfig.get_helm_set_env_args(kubedev, env_accessor, variable_overrides=variable_overrides, cmdline_as_list = False)
       command = [
           '/bin/sh',
           '-c',
-          f'helm upgrade {release_name} ./helm-chart/ --install --wait ' +
+          f'helm upgrade {release_name} ./{helm_chart} --install --wait ' +
           f'--kubeconfig {kubeconfig} {self._get_kubecontext_arg(env_accessor)} ' +
           f'--set KUBEDEV_TAG="{tag}"' +
           envs['cmdline']
@@ -475,15 +494,17 @@ class Kubedev:
 
   def template(self, configFileName, shell_executor=RealShellExecutor(), env_accessor=RealEnvAccessor(), file_accessor=RealFileAccessor()):
     return self.template_from_config(
-        self._load_config(configFileName), shell_executor, env_accessor, file_accessor)
+        self._load_config(configFileName), dict(), shell_executor, env_accessor, file_accessor)
 
   def template_from_config(self,
                            kubedev,
-                           shell_executor,
+                           variable_overrides=dict(),
+                           shell_executor=RealShellExecutor(),
                            env_accessor=RealEnvAccessor(),
                            file_accessor=RealFileAccessor(),
                            get_output=False):
     return self._template(kubedev,
+                                    variable_overrides,
                                     shell_executor,
                                     env_accessor,
                                     file_accessor,
@@ -497,7 +518,7 @@ class Kubedev:
     release_name = KubedevConfig.get_helm_release_name(kubedev)
     kubeconfig = KubedevConfig.get_kubeconfig_path(env_accessor, file_accessor)
     tag = KubedevConfig.get_tag(env_accessor)
-    return self._deploy(kubedev, kubeconfig, tag, release_name, None, shell_executor, env_accessor, file_accessor)
+    return self._deploy(kubedev, kubeconfig, tag, release_name, None, dict(), shell_executor, env_accessor, file_accessor)
 
   def _create_docker_config(self, file_accessor, env_accessor):
     envCi = env_accessor.getenv('CI')
@@ -531,7 +552,7 @@ class Kubedev:
     return self.build_from_config(
         self._load_config(configFileName), container=container, force_tag=None, file_accessor=file_accessor, shell_executor=shell_executor, env_accessor=env_accessor)
 
-  def build_from_config(self, kubedev, container, force_tag, file_accessor, shell_executor, env_accessor):
+  def build_from_config(self, kubedev, container, force_tag, file_accessor, shell_executor, env_accessor) -> int:
     if file_accessor is not None:
       self._create_docker_config(file_accessor, env_accessor)
     images = KubedevConfig.get_images(kubedev, env_accessor)
@@ -617,9 +638,14 @@ class Kubedev:
                         shell,
                         env_accessor):
     polarisConfigFile = self._load_polaris_config(kubedev, downloader, file_accessor, env_accessor)
+    if 'securityChecks' in kubedev and 'variables' in kubedev['securityChecks']:
+      overrideVariables = kubedev['securityChecks']['variables']
+    else:
+      overrideVariables = dict()
 
     k8s_spec        = self.template_from_config(
                         kubedev=kubedev,
+                        variable_overrides=overrideVariables,
                         shell_executor=shell,
                         env_accessor=env_accessor,
                         get_output=True)
@@ -669,7 +695,7 @@ class Kubedev:
         result = False
 
     envs = KubedevConfig.get_all_envs(kubedev, build=is_command(
-        'build'), container=is_command('deploy') or is_command('template')).keys()
+        'build'), container=is_command('deploy') or is_command('template'), generics=is_command('deploy') or is_command('template')).keys()
     for env in sorted(envs):
       if isinstance(env_accessor.getenv(env), type(None)):
         printer.print(
@@ -715,7 +741,7 @@ class Kubedev:
           '/bin/sh',
           '-c',
           f"docker run --interactive {interactive_flags}--rm " +
-          KubedevConfig.get_docker_run_volumes(image, file_accessor, shell_executor) +
+          KubedevConfig.get_docker_run_volumes(Kubedev._field_optional(image, 'volumes', dict()), file_accessor, shell_executor) +
           KubedevConfig.get_docker_run_ports(image) +
           runEnvArgs +
           f"{image['imageNameTagless']}:{currentTag}"
@@ -737,6 +763,7 @@ class Kubedev:
     rawImage: str,
     images: dict,
     variables: dict,
+    volumes: dict,
     cmd: list,
     env_accessor: object,
     shell_executor: object,
@@ -755,16 +782,28 @@ class Kubedev:
         network,
         name,
         image,
-        KubedevConfig.get_docker_run_volumes_list(imageDef, file_accessor, shell_executor) + \
+        KubedevConfig.get_docker_run_volumes_list(Kubedev._field_optional(imageDef, 'volumes', dict()), file_accessor, shell_executor) + \
         requiredEnvForwards + \
         functools.reduce(operator.concat, [["--env", f'{varName}="{varValue}"'] for varName, varValue in variables.items()], []) + \
         functools.reduce(operator.concat, [["--publish", str(port)] for port in ports], []),
         dict(),
+        volumes,
         cmd,
-        shell_executor)
+        shell_executor,
+        file_accessor)
 
 
-  def _run_docker_detached_impl(self, network: str, name: str, image: str, additionalArgs: list, shellEnvs: dict, cmd: list, shell_executor: object):
+  def _run_docker_detached_impl(
+    self,
+    network: str,
+    name: str,
+    image: str,
+    additionalArgs: list,
+    shellEnvs: dict,
+    volumes: dict,
+    cmd: list,
+    shell_executor: object,
+    file_accessor: object):
       print(f'Running detached: {name} (image: {image})')
       cmdRm = ["docker", "rm", "--force", name]
       shell_executor.execute(cmdRm, check=False) # To be sure, first try to delete the container that we want to create
@@ -778,6 +817,7 @@ class Kubedev:
           "--network", network,
           "--name", name,
           "--rm"] + \
+          KubedevConfig.get_docker_run_volumes_list(volumes, file_accessor, shell_executor) + \
           additionalArgs + \
           [image] + cmd)]
       dockerIdRaw = shell_executor.get_output(cmdCreate, envVars=shellEnvs, check=False)
@@ -830,6 +870,7 @@ class Kubedev:
   def system_test(self,
                   configFileName: str,
                   appName: str,
+                  reuseCluster: str,
                   file_accessor=RealFileAccessor(),
                   env_accessor=RealEnvAccessor(),
                   shell_executor=RealShellExecutor(),
@@ -838,13 +879,14 @@ class Kubedev:
     return self.system_test_from_config(
       self._load_config(configFileName, file_accessor),
       appName,
+      reuseCluster,
       file_accessor=file_accessor,
       env_accessor=env_accessor,
       shell_executor=shell_executor,
       tag_generator=tag_generator,
       sleeper=sleeper)
 
-  def system_test_from_config(self, kubedev, appName: str, file_accessor, env_accessor, shell_executor, tag_generator, sleeper) -> bool:
+  def system_test_from_config(self, kubedev, appName: str, reuseCluster: str, file_accessor, env_accessor, shell_executor, tag_generator, sleeper) -> bool:
       '''
       Runs the system tests for an app as defined in the kubedev config.
 
@@ -856,6 +898,7 @@ class Kubedev:
 
       self._create_docker_config(file_accessor, env_accessor)
 
+      serviceName = kubedev['name']
       images = KubedevConfig.get_images(kubedev, env_accessor)
       apps = KubedevConfig.get_all_app_definitions(kubedev)
       if not appName in apps:
@@ -881,7 +924,6 @@ class Kubedev:
       containerDir = f"./systemTests/{appName}/"
       uuid = tag_generator.tag()
       tag = f"local-{appName}-system-tests-{uuid}"
-      network = tag
       appType = app['type']
       cmdBuild = [
         "/bin/sh",
@@ -896,37 +938,57 @@ class Kubedev:
           return False
       # Step #1.1: For CronJobs: Build and push all apps
       if appType == 'cronjobs':
-        for app in apps:
-          self.build_from_config(kubedev, appName, uuid, file_accessor, shell_executor, env_accessor)
-          self._push_image(f"{images[appName]['imageNameTagless']}:{uuid}", shell_executor)
+        for appName, app in apps.items():
+          if self.build_from_config(kubedev, appName, uuid, file_accessor, shell_executor, env_accessor) != 0:
+            return False
+          if self._push_image(f"{images[appName]['imageNameTagless']}:{uuid}", shell_executor) != 0:
+            return False
 
       # Step #2: Create the docker network, or spin up a Kind cluster:
       if appType == 'cronjobs':
           file_accessor.mkdirhier('.kubedev')
-          serviceName = kubedev['name']
-          clusterName = f'kind-{serviceName}-{uuid}'
-          clusterConfig = f'.kubedev/kind_config_{serviceName}-{uuid}'
+          if reuseCluster is not None:
+            clusterName = reuseCluster
+            network = reuseCluster
+            cmdGetKindClusters = [ 'kind', 'get', 'clusters' ]
+            if reuseCluster not in shell_executor.get_output(cmdGetKindClusters):
+              createCluster = True
+            else:
+              createCluster = False
+              clusterConfig = f'.kubedev/kind_config_{reuseCluster}'
+              shell_executor.execute(['kind', 'export', 'kubeconfig', '--name', reuseCluster, '--kubeconfig', clusterConfig])
+              clusterConfigContent = Kubedev._rewrite_kubeconfig_to(clusterConfig, f"{clusterName}-control-plane", 6443, file_accessor=file_accessor)
+          else:
+            createCluster = True
+            clusterName = f'kind-{serviceName}-{uuid}'
+            network = tag
 
-          cmdKindCreateCluster = [
-              'kind',
-              'create',
-              'cluster',
-              '--kubeconfig',
-              clusterConfig,
-              '--wait',
-              '10m',
-              '--name',
-              clusterName
-          ]
-          additionalVars = {
-              'KIND_EXPERIMENTAL_DOCKER_NETWORK': network
-          }
-          # Spin up a new one-node Kubernetes-in-Docker cluster via `kind` (https://kind.sigs.k8s.io/)
-          if shell_executor.execute(cmdKindCreateCluster, additionalVars) != 0:
-            return False
-          clusterConfigContent = Kubedev._rewrite_kubeconfig_to(clusterConfig, f"{clusterName}-control-plane", 6443, file_accessor=file_accessor)
+          if createCluster:
+            clusterConfig = f'.kubedev/kind_config_{serviceName}-{uuid}'
+            cmdKindCreateCluster = [
+                'kind',
+                'create',
+                'cluster',
+                '--kubeconfig',
+                clusterConfig,
+                '--wait',
+                '10m',
+                '--name',
+                clusterName
+            ]
+            additionalVars = {
+                'KIND_EXPERIMENTAL_DOCKER_NETWORK': network
+            }
+            # Spin up a new one-node Kubernetes-in-Docker cluster via `kind` (https://kind.sigs.k8s.io/)
+            if shell_executor.execute(cmdKindCreateCluster, additionalVars) != 0:
+              return False
+            clusterConfigContent = Kubedev._rewrite_kubeconfig_to(clusterConfig, f"{clusterName}-control-plane", 6443, file_accessor=file_accessor)
+
+          wslClusterConfig = KubedevConfig.wsl_normalize(clusterConfig, file_accessor, shell_executor)
       elif appType == 'deployments':
+          network = tag
           clusterConfig = None
+          wslClusterConfig = None
           cmdNetworkCreate = ["docker", "network", "create", network]
           if shell_executor.execute(cmdNetworkCreate, check=False) != 0:
               return False
@@ -945,6 +1007,7 @@ class Kubedev:
                   serviceKey,
                   images,
                   {**globalVariables, **self._field_optional(service, 'variables', dict())},
+                  self._field_optional(service, 'volumes', dict()),
                   self._field_optional(service, 'cmd', []),
                   env_accessor,
                   shell_executor,
@@ -955,7 +1018,7 @@ class Kubedev:
             variables = {
               **variables,
               "KUBEDEV_SYSTEMTEST_DAEMON_APIKEY": runCronJobApiKey,
-              "KUBEDEV_SYSTEMTEST_DAEMON_ENDPOINT": f"http://kubedev-run-cronjob-api:5000/execute"
+              "KUBEDEV_SYSTEMTEST_DAEMON_ENDPOINT": f"kubedev-run-cronjob-api:5000"
             }
 
             # Step #3.1: For CronJobs: Start the 'kubedev-run-cronjob-api' service
@@ -975,53 +1038,74 @@ class Kubedev:
                 {
                   "KUBEDEV_SYSTEMTEST_DAEMON_KUBECONFIG": clusterConfigContent
                 },
+                dict(),
                 [],
-                shell_executor
+                shell_executor,
+                file_accessor
               )
             ]
-            # Step #3.2: For CronJobs: Init helm, set docker registry secret and deploy this helm-chart
-            wslClusterConfig = KubedevConfig.wsl_normalize(clusterConfig, file_accessor, shell_executor)
-            if not KubernetesTools.apply(tag, wslClusterConfig, KubernetesTools.get_tiller_rbac_setup(), shell_executor):
-              return False
-
-            if KubernetesTools.kubectl(
-              tag,
-              wslClusterConfig,
-              {"DOCKER_AUTH_CONFIG"},
-              ["create",
-                "secret",
-                "generic",
-                kubedev['imagePullSecrets'],
-                "--type",
-                "kubernetes.io/dockerconfigjson",
-                '--from-literal=.dockerconfigjson="${DOCKER_AUTH_CONFIG}"'],
-              shell_executor) != 0:
-              return False
-
-            cmdHelmInit = [
-              'docker',
-              'run',
-              '-i',
-              '--rm',
-              '--network',
-              tag,
-              '--volume',
-              f'{wslClusterConfig}:/tmp/kube_config',
-              'alpine/helm:2.16.9',
-              '--kubeconfig',
-              '/tmp/kube_config',
-              'init',
-              '--service-account',
-              'tiller'
-            ]
-            if shell_executor.execute(cmdHelmInit, check=False) != 0:
+            if createCluster:
+              # Step #3.2: For CronJobs: Init helm, set docker registry secret
+              if not KubernetesTools.apply(network, wslClusterConfig, KubernetesTools.get_tiller_rbac_setup(), shell_executor):
                 return False
-            timeout = 600
-            print(f'{colorama.Fore.YELLOW}Waiting {timeout} seconds for Tiller deployment to become ready...')
-            if not KubernetesTools.wait_for_deployment(tag, wslClusterConfig, "kube-system", "tiller-deploy", timeout, shell_executor, sleeper):
-              print(f'{colorama.Fore.RED}Tiller deployment did not become ready within timeout of {timeout} seconds :-(')
-              return False
-            if self._deploy(kubedev, clusterConfig, uuid, tag, tag, shell_executor, env_accessor, file_accessor) != 0:
+
+              if KubernetesTools.kubectl(
+                network,
+                wslClusterConfig,
+                {"DOCKER_AUTH_CONFIG"},
+                ["create",
+                  "secret",
+                  "generic",
+                  kubedev['imagePullSecrets'],
+                  "--type",
+                  "kubernetes.io/dockerconfigjson",
+                  '--from-literal=.dockerconfigjson="${DOCKER_AUTH_CONFIG}"'],
+                shell_executor) != 0:
+                return False
+
+              cmdHelmInit = [
+                'docker',
+                'run',
+                '-i',
+                '--rm',
+                '--network',
+                network,
+                '--volume',
+                f'{wslClusterConfig}:/tmp/kube_config',
+                'alpine/helm:2.16.9',
+                '--kubeconfig',
+                '/tmp/kube_config',
+                'init',
+                '--service-account',
+                'tiller'
+              ]
+              if shell_executor.execute(cmdHelmInit, check=False) != 0:
+                  return False
+              timeout = 600
+              print(f'{colorama.Fore.YELLOW}Waiting {timeout} seconds for Tiller deployment to become ready...')
+              if not KubernetesTools.wait_for_deployment(network, wslClusterConfig, "kube-system", "tiller-deploy", timeout, shell_executor, sleeper):
+                print(f'{colorama.Fore.RED}Tiller deployment did not become ready within timeout of {timeout} seconds :-(')
+                return False
+
+            # Step #3.3: Deploy the clusterInitChart (optional) and this helm-chart
+            if 'clusterInitChart' in systemTestDefinition:
+              clusterInitChartPath = systemTestDefinition['clusterInitChart']
+              print(f'{colorama.Fore.YELLOW}A cluster initialization helm chart is specified at {clusterInitChartPath}, installing now...')
+              if self._deploy(
+                  kubedev,
+                  clusterConfig,
+                  uuid,
+                  f'prepare-{serviceName}',
+                  network,
+                  globalVariables,
+                  shell_executor,
+                  env_accessor,
+                  file_accessor,
+                  helm_chart=clusterInitChartPath,
+                  add_unknown_variable_overrides=True) != 0:
+                return False
+
+            if self._deploy(kubedev, clusterConfig, uuid, serviceName, network, globalVariables, shell_executor, env_accessor, file_accessor) != 0:
               return False
 
           numSleepSeconds = 5
@@ -1046,7 +1130,8 @@ class Kubedev:
               "--name", f"{appName}-system-tests-{uuid}",
               "--interactive"] + \
               Kubedev._build_cluster_config_mounts(clusterConfig, file_accessor=file_accessor, shell_executor=shell_executor) + \
-              functools.reduce(operator.concat, [["--env", f'{envName}="${{{attribs["targetName"]}}}"'] for envName, attribs in filteredRequiredEnvs.items()], []) + \
+              KubedevConfig.get_docker_run_volumes_list(Kubedev._field_optional(testContainer, 'volumes', dict()), file_accessor, shell_executor) + \
+              functools.reduce(operator.concat, [["--env", f'{envName}="{attribs["targetName"]}"'] for envName, attribs in filteredRequiredEnvs.items()], []) + \
               functools.reduce(operator.concat, [["--env", f'{varName}="{varValue}"'] for varName, varValue in variables.items()], []) + \
               [tag])]
           if shell_executor.execute(cmdRunSystemTests, envVars=additionalEnvs, check=False) == 0:
@@ -1074,17 +1159,24 @@ class Kubedev:
                   shell_executor.execute(cmdRm, check=False)
 
           # Cleanup #2: Remove the docker network / Kind cluster
-          if appType == 'cronjobs':
-              cmdKindDeleteCluster = [
-                  'kind',
-                  'delete',
-                  'cluster',
-                  '--name',
-                  clusterName
-              ]
-              shell_executor.execute(cmdKindDeleteCluster)
-          cmdNetworkRm = ["docker", "network", "rm", tag]
-          shell_executor.execute(cmdNetworkRm, check=False)
+          if reuseCluster is None:
+            if appType == 'cronjobs':
+                cmdKindDeleteCluster = [
+                    'kind',
+                    'delete',
+                    'cluster',
+                    '--name',
+                    clusterName
+                ]
+                shell_executor.execute(cmdKindDeleteCluster)
+            cmdNetworkRm = ["docker", "network", "rm", tag]
+            shell_executor.execute(cmdNetworkRm, check=False)
+
+          if reuseCluster is not None:
+            print(f'{colorama.Fore.YELLOW}(ℹ) Keeping network {network} and cluster {reuseCluster} alive for further re-use.')
+            print()
+            print(f'Run this command to access your cluster:')
+            print(f'  docker run -it --rm --network {network} --volume "{wslClusterConfig}:/tmp/kube_config" bitnami/kubectl --kubeconfig /tmp/kube_config <your-commands>')
 
           if result:
               print()
