@@ -17,10 +17,14 @@ class KubedevConfig:
     }
 
   @staticmethod
-  def get_all_envs(kubedev, build, container):
+  def get_all_envs(kubedev, build, container, generics):
     envs = KubedevConfig.load_envs(
         kubedev, build=build, container=container)
-    for appType in ['deployments', 'cronjobs']:
+    if generics:
+      appTypes = ['deployments', 'cronjobs', 'generic']
+    else:
+      appTypes = ['deployments', 'cronjobs']
+    for appType in appTypes:
       if appType in kubedev:
         for (_, deployment) in kubedev[appType].items():
           deploymentEnvs = KubedevConfig.load_envs(
@@ -37,7 +41,7 @@ class KubedevConfig:
         return dict()
 
     return {
-      key: definition for key, definition in {**if_exists(kubedev, 'deployments'), **if_exists(kubedev, 'cronjobs'), **if_exists(kubedev, 'generic')}.items()
+      key: definition for key, definition in {**if_exists(kubedev, 'deployments'), **if_exists(kubedev, 'cronjobs')}.items()
     }
 
   @staticmethod
@@ -53,27 +57,32 @@ class KubedevConfig:
 
     sortedEnvs = dict(sorted(envs.items()))
     return (
-      {name: {**attribs, **{'targetName': env_name(name, attribs)}} for name, attribs in sortedEnvs.items()},
+      {name: {**attribs, **{'rawTargetName': env_name(name, attribs), 'targetName': f'${{{env_name(name, attribs)}}}'}} for name, attribs in sortedEnvs.items()},
       {env_name(name, attribs): b64encode(env_accessor.getenv(name, default="").encode('utf-8')) for name, attribs in sortedEnvs.items() if is_base64(attribs)}
     )
 
   @staticmethod
-  def get_helm_set_env_args(kubedev: dict, env_accessor: object, cmdline_as_list: bool = False) -> dict:
+  def get_helm_set_env_args(kubedev: dict, env_accessor: object, variable_overrides: dict = dict(), add_unknown_variable_overrides: bool = False, cmdline_as_list: bool = False) -> dict:
     '''
     Returns shell parameters for helm commands in the form of ``--set <variable>="${<variable>}" ...''
     from a kubedev config.
     '''
-    (envs, extraEnvs) = KubedevConfig.prepare_envs(KubedevConfig.get_all_envs(kubedev, False, True), env_accessor=env_accessor)
+    (envs, extraEnvs) = KubedevConfig.prepare_envs(KubedevConfig.get_all_envs(kubedev, False, True, True), env_accessor=env_accessor)
+    for var, value in variable_overrides.items():
+      if (var in envs) or add_unknown_variable_overrides:
+        envs[var] = {
+          'targetName': value
+        }
 
     if len(envs) > 0:
       if cmdline_as_list:
         return {
-          'cmdline': [arg for arglist in [['--set', f'{name}="${{{attribs["targetName"]}}}"'] for name, attribs in envs.items()] for arg in arglist],
+          'cmdline': [arg for arglist in [['--set', f'{name}="{attribs["targetName"]}"'] for name, attribs in envs.items()] for arg in arglist],
           'envs': extraEnvs
         }
       else:
         return {
-          'cmdline': ' ' + ' '.join([f'--set {name}="${{{attribs["targetName"]}}}"' for name, attribs in envs.items()]),
+          'cmdline': ' ' + ' '.join([f'--set {name}="{attribs["targetName"]}"' for name, attribs in envs.items()]),
           'envs': extraEnvs
         }
     else:
@@ -184,7 +193,7 @@ class KubedevConfig:
     """
     (envs, extraEnvs) = KubedevConfig.prepare_envs(image['buildEnvs'], env_accessor=env_accessor)
     return (
-      " ".join([f'--build-arg {name}="${{{attribs["targetName"]}}}"' for name, attribs in sorted(envs.items())]) + " ",
+      " ".join([f'--build-arg {name}="{attribs["targetName"]}"' for name, attribs in sorted(envs.items())]) + " ",
       extraEnvs
     )
 
@@ -197,7 +206,7 @@ class KubedevConfig:
     """
     (envs, extraEnvs) = KubedevConfig.prepare_envs(image['containerEnvs'], env_accessor)
     return (
-      " ".join([f'--env {name}="${{{attribs["targetName"]}}}"' for name, attribs in sorted(envs.items())]) + " ",
+      " ".join([f'--env {name}="{attribs["targetName"]}"' for name, attribs in sorted(envs.items())]) + " ",
       extraEnvs
     )
 
@@ -212,11 +221,11 @@ class KubedevConfig:
 
 
   @staticmethod
-  def get_docker_run_volumes_list(image, file_accessor, shell_executor):
+  def get_docker_run_volumes_list(volumes: dict, file_accessor, shell_executor) -> list:
     """
     Returns a string with all "--volume ..." parameters to the "docker run ..." call.
 
-    :param image: One entry returned from KubedevConfig.get_images()
+    :param volumes: A dict with volume definitions.
     """
     def get_path(hostPath, containerPathSpec) -> str:
       if type(containerPathSpec) is dict:
@@ -241,12 +250,11 @@ class KubedevConfig:
       else:
         raise Exception(f'Volume specification must either be a string, or an object with an "path" property, but is {type(containerPathSpec)} instead.')
 
-    volumes = image["volumes"] if "volumes" in image else dict()
     return list(itertools.chain(*[["--volume", get_path(hostPath, containerPath)] for hostPath, containerPath in volumes.items()]))
 
   @staticmethod
-  def get_docker_run_volumes(image, file_accessor, shell_executor):
-    args = KubedevConfig.get_docker_run_volumes_list(image, file_accessor, shell_executor)
+  def get_docker_run_volumes(volumes: dict, file_accessor, shell_executor) -> str:
+    args = KubedevConfig.get_docker_run_volumes_list(volumes, file_accessor, shell_executor)
     return " ".join(args) + (" " if len(args) > 0 else "")
 
   @staticmethod
